@@ -35,6 +35,17 @@ gen_crawl_event!(crawl_l2_event_inverse_swap, HuobiInverseSwapWSClient, MessageT
 #[rustfmt::skip]
 gen_crawl_event!(crawl_l2_event_option, HuobiOptionWSClient, MessageType::L2Event, subscribe_orderbook);
 
+#[rustfmt::skip]
+gen_crawl_event!(crawl_bbo_spot, HuobiSpotWSClient, MessageType::BBO, subscribe_bbo);
+#[rustfmt::skip]
+gen_crawl_event!(crawl_bbo_inverse_future, HuobiFutureWSClient, MessageType::BBO, subscribe_bbo);
+#[rustfmt::skip]
+gen_crawl_event!(crawl_bbo_linear_swap, HuobiLinearSwapWSClient, MessageType::BBO, subscribe_bbo);
+#[rustfmt::skip]
+gen_crawl_event!(crawl_bbo_inverse_swap, HuobiInverseSwapWSClient, MessageType::BBO, subscribe_bbo);
+#[rustfmt::skip]
+gen_crawl_event!(crawl_bbo_option, HuobiOptionWSClient, MessageType::BBO, subscribe_bbo);
+
 #[rustfmt::skip]
 gen_crawl_event!(crawl_ticker_spot, HuobiSpotWSClient, MessageType::Ticker, subscribe_ticker);
 #[rustfmt::skip]
@@ -111,6 +122,24 @@ pub(crate) fn crawl_l2_event(
     }
 }
 
+pub(crate) fn crawl_bbo(
+    market_type: MarketType,
+    symbols: Option<&[String]>,
+    on_msg: Arc<Mutex<dyn FnMut(Message) + 'static + Send>>,
+    duration: Option<u64>,
+) -> Option<std::thread::JoinHandle<()>> {
+    match market_type {
+        MarketType::Spot => crawl_bbo_spot(market_type, symbols, on_msg, duration),
+        MarketType::InverseFuture => {
+            crawl_bbo_inverse_future(market_type, symbols, on_msg, duration)
+        }
+        MarketType::LinearSwap => crawl_bbo_linear_swap(market_type, symbols, on_msg, duration),
+        MarketType::InverseSwap => crawl_bbo_inverse_swap(market_type, symbols, on_msg, duration),
+        MarketType::EuropeanOption => crawl_bbo_option(market_type, symbols, on_msg, duration),
+        _ => panic!("Huobi does NOT have the {} market type", market_type),
+    }
+}
+
 pub(crate) fn crawl_ticker(
     market_type: MarketType,
     symbols: Option<&[String]>,
